#include <stdio.h>
#include "co.h"
#include "cosim_log.h"
#include "Fim.h"

extern void call_accelerator (co_stream output_stream, co_stream input_stream);

//support counting
void hw_support_counting(co_stream fim_in, co_stream fim_out)
{
//Argeumets:

// next_candidate      :    the 2d candidate list pointer
// next_candidate_size :    number of candidate or the row number of next_candidate
// g                   :    generation number or the column number of next_candidate

  //IF_SIM(printf("************* Call hw_support_counting **************\n"));
  int i,j,k,m,n,l,p,q,r;
  int c;
  int t;
  int number_of_reload, candidate_per_load,remainder;
  int g;
  int next_candidate_size;
  int ptr,temp;

  //Mass storage
  int data[LENGTH_OF_MEM];//data storage
  #pragma CO NONRECURSIVE data
  int reg[LENGTH_OF_MEM]; //intermediate support storage
  #pragma CO NONRECURSIVE reg
  int trans[LENGTH_OF_TRANSACTION];//input transaction data storage
  #pragma CO NONRECURSIVE trans
  
  int inSample;
  int outSample;
  int number_of_candidate;
  int number_of_transaction;
  int length_of_transaction;

  IF_SIM (int samplesread = 0;)
	IF_SIM (int sampleswritten = 0;)
	
	IF_SIM ( cosim_logwindow log; )
	IF_SIM ( log = cosim_logwindow_create ("hw_support_counting"); )

  //Memory configuration
 	co_array_config(data,co_kind,"sync");
	co_array_config(reg,co_kind,"sync");
  co_array_config(trans,co_kind,"sync");

  do 
   { // Hardware processes run forever
     
     co_stream_open (fim_in,  O_RDONLY, INT_TYPE(32));
     co_stream_open (fim_out, O_WRONLY, INT_TYPE(32));
       
       // Load number of candidate 
       if(co_stream_read (fim_in, &inSample, sizeof(int32)) == co_err_none)
       {
       IF_SIM (samplesread++;) 
       number_of_candidate = inSample;
       IF_SIM(printf("hw --> number_of_candidate : %d\n", number_of_candidate));
       }
        else
         break;
       // Load generation number
       if( co_stream_read (fim_in, &inSample, sizeof(int32)) == co_err_none)
       {
       IF_SIM (samplesread++;) 
       g = inSample;
       IF_SIM(printf("hw --> generation number : %d\n", g));
       }
       else 
        break;

     // Load number of transaction
       if( co_stream_read (fim_in, &inSample, sizeof(int32)) == co_err_none)
       {
       IF_SIM (samplesread++;) 
       number_of_transaction = inSample;
       IF_SIM(printf("hw --> generation number : %d\n", g));
       }
       else 
        break;

      // Load length of transaction
       if( co_stream_read (fim_in, &inSample, sizeof(int32)) == co_err_none)
       {
       IF_SIM (samplesread++;) 
       length_of_transaction = inSample;
       IF_SIM(printf("hw --> generation number : %d\n", g));
       }
       else 
        break;
        
      // Load candidate into data[] and clear reg[]   
       i = 0;
         do {
           #pragma CO PIPELINE	
           // Fill the Data Memory
           co_stream_read (fim_in, &inSample, sizeof(int32));
           IF_SIM (samplesread++;) 
           IF_SIM(printf("hw --> data[%d] : %d\n", i, inSample));
           data[i++] = inSample;
       
         } while(i < number_of_candidate *g);
       
        j = 0;
         do {
           #pragma CO PIPELINE
   
           // Clear the reg
           reg[j++] = 0;
         } while(j < LENGTH_OF_MEM);
          

          // Read transaction and perform counting
         for(t = 0; t < number_of_transaction; t++)//pointer to transaction number
         {  
           m = 0;
           do {
           #pragma CO PIPELINE	
           // Fill the Transaction register
           co_stream_read (fim_in, &inSample, sizeof(int32));
           IF_SIM (samplesread++;) 
           IF_SIM(printf("hw --> trans[%d] : %d\n", m, inSample));
           trans[m] = inSample;
           m++;
           } while(m < length_of_transaction);
          
  
  
           for(c = 0; c < number_of_candidate; c++)//pointer to candidate
		   {		
             ptr = -1;
			 for(l = 0; l < length_of_transaction; l++)//pointer to item within transaction
 			 {
               k = 0;
               while (k < g)
               {
               #pragma CO PIPELINE
		       #pragma CO set stageDelay 25   
                if ((trans[l] == data[c * g + k]))
                 {
                   if (g == 1)
                    reg[c]++;
                   else 
                   {//increment reg
                     if (k == 0) {
                       reg[c * g] = 1;
                       ptr = 0; 
                     }
                     else {
                       if (k == ptr + 1) {
						reg[c * g + k]++;
						ptr = k;
					   } 
					 }
                   }
                   break;
                 } 
                 else 
                   {
                    k++;}
               } 
             }
           }
           
           // One transaction is done and clear the intermediate reg[] 
           /*
           for(n = 0; n < number_of_candidate*g; n++)
           {
             #pragma CO PIPELINE	
             if((g != 1) && (n % g != (g-1)))//reg store the final count for this transaciont
               reg[n] = 0;
           } */
           
		   if (g != 1) {
			for(n = 0; n < number_of_candidate; n++)
             for (r = 0; r < g - 1; r++) {
			   #pragma CO PIPELINE
			   reg[n * g + r] = 0; 
		      }
            }
         }
         
         //count is ready for output
       
         for(p= 0; p < number_of_candidate; p++) 
         {
           #pragma CO PIPELINE	
           #pragma CO set stageDelay 25  
           //int offset = l * candidate_per_load;
           //count[offset + j]= reg[j * g + g - 1];
           temp = p * g + g - 1;
           outSample  = reg[temp];
           IF_SIM(printf("outSample : %d\n", outSample));
           co_stream_write (fim_out, &outSample, sizeof(int32));
           IF_SIM(sampleswritten++;)
        } 
  
        IF_SIM (cosim_logwindow_fwrite (log, 
              "Closing Support Counting, samples read : %d, samples written : %d\n",
  			      samplesread, sampleswritten);)
  
//      if (co_stream_read (fim_in, &inSample, sizeof(int32)) == co_err_eos)
        if(co_stream_eos(fim_in) == co_err_eos)
       {
          co_stream_close (fim_in);
          co_stream_close (fim_out);
          IF_SIM(break;)        
       }       

     } while (1); 
}



void config_fim (void *arg) {
	int i;
	
	co_stream  to_fim, from_fim;
	co_process cpu_proc, fim_proc;
	
	to_fim   = co_stream_create ("to_fim",   INT_TYPE(32), 16); 
	from_fim = co_stream_create ("from_fim", INT_TYPE(32), 16);
	
	cpu_proc    = co_process_create ("cpu_proc",    (co_function) call_accelerator, 2, to_fim, from_fim);
	fim_proc = co_process_create ("fim_proc", (co_function) hw_support_counting, 2, to_fim, from_fim);
	
	co_process_config (fim_proc, co_loc, "PE0");
}

co_architecture co_initialize () {
	return (co_architecture_create ("fim", "altera_nios2", config_fim, NULL));
}
