-- 
-- Copyright (c) 2002-2010 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
-- 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

library impulse;
use impulse.all;


entity tb_fim is
end tb_fim;

architecture behavior of tb_fim is
  constant NUM_TO_SEND : integer := 32;
  constant CLK_PERIOD : time := 10 ns;
  constant SCLK_PERIOD : time := 10 ns;   -- in a dual-clock design, 'sclk' doesn't have to equal 'clk'

  component fim is
    port (
    reset : in std_ulogic;
    sclk : in std_ulogic;
    clk : in std_ulogic;
    p_cpu_proc_output_stream_en : in std_ulogic;
    p_cpu_proc_output_stream_eos : in std_ulogic;
    p_cpu_proc_output_stream_data : in std_ulogic_vector (31 downto 0);
    p_cpu_proc_output_stream_rdy : out std_ulogic;
    p_cpu_proc_input_stream_en : in std_ulogic;
    p_cpu_proc_input_stream_data : out std_ulogic_vector (31 downto 0);
    p_cpu_proc_input_stream_eos : out std_ulogic;
    p_cpu_proc_input_stream_rdy : out std_ulogic
    );
  end component;

-- Internal Signal Definitions.
  signal reset : std_ulogic;
  signal clk : std_ulogic;
  signal sclk : std_ulogic;
  signal p_cpu_proc_output_stream_rdy : std_ulogic;
  signal p_cpu_proc_output_stream_en : std_ulogic;
  signal p_cpu_proc_output_stream_eos : std_ulogic;
  signal p_cpu_proc_output_stream_idata : std_ulogic_vector (31 downto 0);
  signal p_cpu_proc_input_stream_rdy : std_ulogic;
  signal p_cpu_proc_input_stream_en : std_ulogic;
  signal p_cpu_proc_input_stream_eos : std_ulogic;
  signal p_cpu_proc_input_stream_idata : std_ulogic_vector (31 downto 0);
  signal stream_out_eof : std_ulogic;
  signal p_cpu_proc_output_stream_eof : std_ulogic;

begin
  reset_proc : process
  begin
        reset <= '1';
        wait for 100 ns;
        reset <= '0';
        wait;
  end process;

  clk_proc : process
  begin
        clk <= '0';
        wait for CLK_PERIOD/2;
        clk <= '1';
        wait for CLK_PERIOD/2;
  end process;

  sclk_proc : process
  begin
        sclk <= '0';
        wait for SCLK_PERIOD/2;
        sclk <= '1';
        wait for SCLK_PERIOD/2;
  end process;
-- TBD may not need
  test_proc : process
  begin
        stream_out_eof <= '0';
        wait for 1 us;
        stream_out_eof <= '1';  -- cause output file to be closed
        wait; -- wait forever
  end process;

  p_cpu_proc_output_stream: entity work.co_trace_stream_read_file
    generic map (
      datawidth => 32,
      inputFilename => "../to_fim.co_trace_strm"
    )
    port map (
      reset => reset,
      clk => clk,
      output_en => p_cpu_proc_output_stream_en,
      output_rdy => p_cpu_proc_output_stream_rdy,
      output_eos => p_cpu_proc_output_stream_eos,
      output_eof => p_cpu_proc_output_stream_eof,
      output_data => p_cpu_proc_output_stream_idata
    );

  p_cpu_proc_input_stream: entity work.co_trace_stream_write_file
    generic map (
      datawidth => 32,
      outputFilename => "from_fim.co_trace_strm"
    )
    port map (
      reset => reset,
      clk => clk,
      input_en => p_cpu_proc_input_stream_en,
      input_rdy => p_cpu_proc_input_stream_rdy,
      input_eos => p_cpu_proc_input_stream_eos,
      input_eof => stream_out_eof,
      input_data => p_cpu_proc_input_stream_idata
    );

  fim_0: fim
    port map (
      reset,
      sclk,
      clk,
      p_cpu_proc_output_stream_en,
      p_cpu_proc_output_stream_eos,
      p_cpu_proc_output_stream_idata,
      p_cpu_proc_output_stream_rdy,
      p_cpu_proc_input_stream_en,
      p_cpu_proc_input_stream_idata,
      p_cpu_proc_input_stream_eos,
      p_cpu_proc_input_stream_rdy);

end;
