#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include "co.h"

#include "Fim.h"
#include "Fim_lib.h"
#include "global.h"


#if defined(IMPULSE_C_TARGET)
#include <sys/time.h>
#else
#include "cosim_log.h"
#endif

struct timeval tv;
double endtime, starttime, elapsetime;

#if defined(IMPULSE_C_TARGET)
  double elapsedtime_hw, elapsedtime_sw;
#endif


// Impulse declarations
extern co_architecture co_initialize(void *);

int main(void)
{

	// Impulse C variables
    int c;
	co_architecture my_arch;

    int i;
	
    maximum_candidate = (NUMBER_OF_ITEM * NUMBER_OF_ITEM * NUMBER_OF_ITEM) / 2;
    maximum_generation = 10;
   
    int **candidate;
    int **next_candidate;// The return result of generate_candidate 
     
    printf("Program parameters for this run:\n");
    printf("Number of items: %d\n", NUMBER_OF_ITEM);
    printf("Number of transaction: %d\n", NUMBER_OF_TRANSACTION);
    printf("Length of transaction: %d\n", LENGTH_OF_TRANSACTION);
    printf("Threshold: %d\n", THRESHOLD);
    
    //Initialize the candidate memory
    
    candidate =  malloc(sizeof (int *) * maximum_candidate );
    if(candidate)
    {
      for(i= 0;i< maximum_candidate; i++)
      {
       candidate[i] = malloc(sizeof (int) * maximum_generation);
      }
    }
    else
    {
      printf("candidate memory allocation error\n");
      exit (1);
    } 
    
    //Initialize the next_candidate memory

    next_candidate = malloc(sizeof (int *) * maximum_candidate );
    if(next_candidate)
    {
      for(i= 0;i< maximum_candidate; i++)
      {
       next_candidate[i] = malloc(sizeof (int) * maximum_generation);
      }
    }
   else
   {
     printf("next_candidate memory allocation error\n");
     exit (1);
   } 
  
   candidate_size = number_of_item;

   while(candidate_size !=0 )
   { 
     printf("Start the %d generation\n",g);
     //Time measuring
     //#ifdef TIMING
     #if defined(IMPULSE_C_TARGET)
     gettimeofday( &tv, NULL );
     starttime = tv.tv_sec+tv.tv_usec*0.000001;
     #endif

     generate_candidate(g,candidate,next_candidate);
     //#ifdef TIMING
     #if defined(IMPULSE_C_TARGET)
     gettimeofday( &tv, NULL );
     endtime = tv.tv_sec+tv.tv_usec*0.000001;
     elapsetime= endtime - starttime;
     printf(" Elaplse time is %.5f second;\n",elapsetime);
     #endif
     printf(" Candidate generation done, the next candidate size is: %d\n",next_candidate_size);
     
     if(next_candidate_size != 0)
     { 
       int count[next_candidate_size];
       //#ifdef TIMING
       #if defined(IMPULSE_C_TARGET)
       gettimeofday( &tv, NULL );
       starttime = tv.tv_sec+tv.tv_usec*0.000001;
       #endif
       support_counting(next_candidate,g,count);
       //#ifdef TIMING
       #if defined(IMPULSE_C_TARGET)
       gettimeofday( &tv, NULL );
       endtime = tv.tv_sec+tv.tv_usec*0.000001;
       elapsetime= endtime - starttime;
       //store the time to global spaces
       sw_time_elapsed[g]=elapsetime;
       printf(" Elaplse time is %.5f second\n",elapsetime);
       #endif
       printf(" Support counting done! \n");
       //#ifdef TIMING
       #if defined(IMPULSE_C_TARGET)
       gettimeofday( &tv, NULL );
       starttime = tv.tv_sec+tv.tv_usec*0.000001;
       #endif
       next_candidate_size =prune_candidate(next_candidate,g,count,THRESHOLD);
       //#ifdef TIMING
       #if defined(IMPULSE_C_TARGET)
       gettimeofday( &tv, NULL );
       endtime = tv.tv_sec+tv.tv_usec*0.000001;
       elapsetime= endtime - starttime;
       printf(" Elaplse time is %.5f second\n",elapsetime);
       #endif
       printf(" Candidate pruning done! There are %d frequent item\n",next_candidate_size);
     }
     
     int idx,i;  
     for( idx = 0; idx < next_candidate_size; idx++)
       for ( i = 0; i < g; i++)
         candidate[idx][i] = next_candidate[idx][i];

     candidate_size = next_candidate_size;

     g++;
   }

	// Running on hardware
	my_arch = co_initialize (NULL);
	co_execute (my_arch);	
   
  for(i=0;i<g-1;i++)
    {
    printf("The speedup for support counting of %d generation is %.2f \n",i+1,sw_time_elapsed[i]/hw_time_elapsed[i]);
    }
 	printf ("COMPLETE APPLICATION\r\n");
	printf ("Press Enter to continue...\r\n");
	c = getc(stdin);   

  return 0;
}


void call_accelerator (co_stream output_stream, co_stream input_stream) {

  int i,j,k,m,l;
  int t;
  int number_of_reload, candidate_per_load,remainder;
  g = 1;

  int32 ReturnVal;
  int32 error; //error code for transaction read
  int temp;
 
  int **candidate;
  int **next_candidate;// The return result of generate_candidate 

    co_stream_open (output_stream, O_WRONLY, INT_TYPE(32));
    co_stream_open (input_stream,  O_RDONLY, INT_TYPE(32));
     
    printf("\r\n****************** Call accelerator ***************\r\n");
    printf("Program parameters for this run:\n");
    printf("Number of items: %d\n", NUMBER_OF_ITEM);
    printf("Number of transaction: %d\n", NUMBER_OF_TRANSACTION);
    printf("Length of transaction: %d\n", LENGTH_OF_TRANSACTION);
    printf("Threshold: %d\n", THRESHOLD);

    //Allocate candidates memory
    candidate =  malloc(sizeof (int *) * maximum_candidate );
    if(candidate)
    {
      for(i= 0;i< maximum_candidate; i++)
      {
       candidate[i] = malloc(sizeof (int) * maximum_generation);
      }
    }
    else
    {
      printf("candidate memory allocation error\n");
      exit (1);
    } 
    
    //Allocate Next candidates memory
    next_candidate = malloc(sizeof (int *) * maximum_candidate );
    if(next_candidate)
    {
      for(i= 0;i< maximum_candidate; i++)
      {
       next_candidate[i] = malloc(sizeof (int) * maximum_generation);
      }
    }
    else
    {
      printf("next_candidate memory allocation error\n");
      exit (1);
    } 

   candidate_size = number_of_item;

   while(candidate_size !=0 )
   { 
     printf("Start the %d generation\n",g);

     //Time measuring
     #if defined(IMPULSE_C_TARGET)
     gettimeofday( &tv, NULL );
     starttime = tv.tv_sec+tv.tv_usec*0.000001;
     #endif

     generate_candidate(g,candidate,next_candidate);

     #if defined(IMPULSE_C_TARGET)
     gettimeofday( &tv, NULL );
     endtime = tv.tv_sec+tv.tv_usec*0.000001;
     elapsetime= endtime - starttime;
     printf("Elapse time is %.5f\n",elapsetime);
     #endif

     printf("Candidate generation done, the next candidate size is: %d\n",next_candidate_size);
     
     if (next_candidate_size != 0)
     { 
       int count[next_candidate_size];

       #if defined(IMPULSE_C_TARGET)
       gettimeofday( &tv, NULL );
       starttime = tv.tv_sec+tv.tv_usec*0.000001;
       #endif

       // Number of candidates for each load
       candidate_per_load = LENGTH_OF_MEM / g;

       // load number
       number_of_reload = (next_candidate_size*g) / (candidate_per_load*g);

       // remainder
       remainder = (next_candidate_size) % (candidate_per_load);
       
       // Get Start Timestamp
       #if defined(IMPULSE_C_TARGET)
       printf ("\r\n===================Running the Accelerated Version==================\r\n");
       printf ("--> Begin support counting!\r\n");
       printf ("**********************************************\r\n");
       gettimeofday( &tv, NULL );
       double starttime = tv.tv_sec+tv.tv_usec*0.000001;
       #else
       printf("Begin support Counting!\r\n");
       #endif
   
       for(l = 0;l< number_of_reload+1;l++)//reload pointer
       {
         int number_of_candidate = (l < number_of_reload) ? candidate_per_load : remainder;

         HW_STREAM_WRITE(cpu_proc,output_stream, number_of_candidate);
         printf("...candidate number from sw: %d\r\n", number_of_candidate );

   

         HW_STREAM_WRITE(cpu_proc,output_stream,g); 
         printf("...generation number from sw: %d\r\n", g);
        
         temp = NUMBER_OF_TRANSACTION;    
         HW_STREAM_WRITE(cpu_proc,output_stream,temp); 
         printf("...NUMBER_OF_TRANSACTION from sw: %d\r\n", NUMBER_OF_TRANSACTION);

         temp = LENGTH_OF_TRANSACTION;  
         HW_STREAM_WRITE(cpu_proc,output_stream,temp); 
         printf("...LENGTH_OF_TRANSACTION from sw: %d\r\n", LENGTH_OF_TRANSACTION);

         // Load candidate into data[] and clear reg[]
         for(j= 0;j<number_of_candidate; j++)
         { 
           for (k = 0; k < g; k++)
           {
              HW_STREAM_WRITE(cpu_proc, output_stream, next_candidate[l*candidate_per_load+j][k]);
              //printf("... next candidates from sw: %d\r\n", next_candidate[l*candidate_per_load+j][k]);
              //data[j*g+k]= next_candidate[l*candidate_per_load+j][k];
              //reg[j*g+k] = 0;
           }
         }

         // Read transaction and perform counting
         for(t = 0; t < NUMBER_OF_TRANSACTION; t++)//pointer to transaction number
         {
           // Load trans[] with one transaction data
           for (m = 0; m < LENGTH_OF_TRANSACTION; m++) {
             HW_STREAM_WRITE(cpu_proc,output_stream, array[t][m]);
             //printf("... transaction data from sw: %d\r\n", array[t][m]);
             //trans[m] = array[t][m];
           }
         }

         for(j= 0;j<number_of_candidate; j++)
         { 
            int offset = l * candidate_per_load;
            // Read result
            HW_STREAM_READ(cpu_proc, input_stream, ReturnVal, error);
            //printf("... output count value from hw: %d\r\n", ReturnVal);
            count[offset + j] = ReturnVal;
         }
       }


          // Get Finish Timestamp
          #if defined(IMPULSE_C_TARGET)
          printf ("\r\n--> Commplete support counting!");
          printf ("**********************************************\r\n");
          gettimeofday( &tv, NULL );
          elapsedtime_hw = tv.tv_sec+tv.tv_usec*0.000001 - starttime;
          // Print Accumulated Stats
          printf (", execution time : %f seconds\r\n", elapsedtime_hw);
          #else
          printf ("\r\n--> Complete supporting counting!\r\n");
          printf ("**********************************************\r\n");
          #endif	

  
              //Start of copying
               #if defined(IMPULSE_C_TARGET) 
               gettimeofday( &tv, NULL );
               endtime = tv.tv_sec+tv.tv_usec*0.000001;
               elapsetime= endtime - starttime;
               hw_time_elapsed[g] = elapsetime;
               printf("Elapse time is %.5f\n",elapsetime);
              #endif
               printf(" Support counting done!\n");
               #if defined(IMPULSE_C_TARGET)
               gettimeofday( &tv, NULL );
               starttime = tv.tv_sec+tv.tv_usec*0.000001;
               #endif
               next_candidate_size =prune_candidate(next_candidate,g,count,THRESHOLD);
               #if defined(IMPULSE_C_TARGET)
               gettimeofday( &tv, NULL );
               endtime = tv.tv_sec+tv.tv_usec*0.000001;
               elapsetime= endtime - starttime;
               printf("Elapse time is %.5f\n",elapsetime);
              #endif
               printf(" Candidate pruning done! There are %d frequent item\n",next_candidate_size);
             }
             
             int idx,i;  
             for( idx = 0; idx < next_candidate_size; idx++)
               for ( i = 0; i < g; i++)
                 candidate[idx][i] = next_candidate[idx][i];

             candidate_size = next_candidate_size;

             g++;
    }

		  HW_STREAM_CLOSE (cpu_proc, output_stream);
      HW_STREAM_CLOSE (cpu_proc, input_stream);

}
