-- 
-- Copyright (c) 2002-2010 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
-- 
library ieee;
use ieee.std_logic_1164.all;

use work.avalon_if.all;

entity fim_module is
  port (
    fim_reset : in std_ulogic;
    co_clk : in std_ulogic;
    clk : in std_ulogic;
    p_cpu_proc_output_stream_reset_n : in std_logic;
    p_cpu_proc_output_stream_ce : in std_logic;
    p_cpu_proc_output_stream_read_n : in std_logic;
    p_cpu_proc_output_stream_write_n : in std_logic;
    p_cpu_proc_output_stream_addr : in std_logic_vector (1 downto 0);
    p_cpu_proc_output_stream_wdata : in std_logic_vector (31 downto 0);
    p_cpu_proc_output_stream_rdata : out std_logic_vector (31 downto 0);
    p_cpu_proc_input_stream_reset_n : in std_logic;
    p_cpu_proc_input_stream_ce : in std_logic;
    p_cpu_proc_input_stream_read_n : in std_logic;
    p_cpu_proc_input_stream_write_n : in std_logic;
    p_cpu_proc_input_stream_addr : in std_logic_vector (1 downto 0);
    p_cpu_proc_input_stream_wdata : in std_logic_vector (31 downto 0);
    p_cpu_proc_input_stream_rdata : out std_logic_vector (31 downto 0));
end;

architecture impl of fim_module is
  component fim is
    port (
    reset : in std_ulogic;
    sclk : in std_ulogic;
    clk : in std_ulogic;
    p_cpu_proc_output_stream_en : in std_ulogic;
    p_cpu_proc_output_stream_eos : in std_ulogic;
    p_cpu_proc_output_stream_data : in std_ulogic_vector (31 downto 0);
    p_cpu_proc_output_stream_rdy : out std_ulogic;
    p_cpu_proc_input_stream_en : in std_ulogic;
    p_cpu_proc_input_stream_data : out std_ulogic_vector (31 downto 0);
    p_cpu_proc_input_stream_eos : out std_ulogic;
    p_cpu_proc_input_stream_rdy : out std_ulogic
    );
  end component;

  signal sclk : std_logic;
  signal p_cpu_proc_output_stream_rdy : std_ulogic;
  signal p_cpu_proc_output_stream_en : std_ulogic;
  signal p_cpu_proc_output_stream_eos : std_ulogic;
  signal p_cpu_proc_output_stream_idata : std_ulogic_vector (31 downto 0);
  signal p_cpu_proc_input_stream_rdy : std_ulogic;
  signal p_cpu_proc_input_stream_en : std_ulogic;
  signal p_cpu_proc_input_stream_eos : std_ulogic;
  signal p_cpu_proc_input_stream_idata : std_ulogic_vector (31 downto 0);
begin
  sclk <= clk;
  p_cpu_proc_output_stream_if: avalon_to_stream
    generic map (
      datawidth => 32
    )
    port map (
      clk => sclk,
      avalon_addr => p_cpu_proc_output_stream_addr,
      avalon_ce => p_cpu_proc_output_stream_ce,
      avalon_read_n => p_cpu_proc_output_stream_read_n,
      avalon_reset_n => p_cpu_proc_output_stream_reset_n,
      avalon_write_n => p_cpu_proc_output_stream_write_n,
      avalon_wdata => p_cpu_proc_output_stream_wdata,
      avalon_rdata => p_cpu_proc_output_stream_rdata,
      stream_en => p_cpu_proc_output_stream_en,
      stream_rdy => p_cpu_proc_output_stream_rdy,
      stream_eos => p_cpu_proc_output_stream_eos,
      stream_data => p_cpu_proc_output_stream_idata
    );

  p_cpu_proc_input_stream_if: stream_to_avalon
    generic map (
      datawidth => 32
    )
    port map (
      clk => sclk,
      avalon_addr => p_cpu_proc_input_stream_addr,
      avalon_ce => p_cpu_proc_input_stream_ce,
      avalon_read_n => p_cpu_proc_input_stream_read_n,
      avalon_reset_n => p_cpu_proc_input_stream_reset_n,
      avalon_write_n => p_cpu_proc_input_stream_write_n,
      avalon_wdata => p_cpu_proc_input_stream_wdata,
      avalon_rdata => p_cpu_proc_input_stream_rdata,
      stream_en => p_cpu_proc_input_stream_en,
      stream_rdy => p_cpu_proc_input_stream_rdy,
      stream_eos => p_cpu_proc_input_stream_eos,
      stream_data => p_cpu_proc_input_stream_idata
    );

  fim_0: fim
    port map (
      fim_reset,
      sclk,
      clk,
      p_cpu_proc_output_stream_en,
      p_cpu_proc_output_stream_eos,
      p_cpu_proc_output_stream_idata,
      p_cpu_proc_output_stream_rdy,
      p_cpu_proc_input_stream_en,
      p_cpu_proc_input_stream_idata,
      p_cpu_proc_input_stream_eos,
      p_cpu_proc_input_stream_rdy);

end;
