/*********************************************************************
* Copyright (c) 2004, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co_stream.c: Stream functions.
*
*********************************************************************/

#include "system.h"

#include <malloc.h>
#include <string.h>

#include "co.h"


co_stream co_stream_create(const char * name, co_type datatype, unsigned int buffer_depth)
{
	co_stream stream = malloc(sizeof(co_stream_t));	
	if ( stream != NULL ) {
		stream->name = strdup(name);
		stream->datatype = datatype;
		stream->buffer_depth = buffer_depth;
		stream->direction = CO_NO_DIRECTION;
		stream->io = NULL;
	}

	return stream;
}

void co_stream_attach(co_stream stream, void * io, co_stream_direction dir)
{
	if ( stream != NULL ) {
	    stream->io=io;
    	stream->direction=dir;
    }
}

co_error co_stream_close(co_stream stream) 
{
	if ( stream != NULL ) {
	    nr_impc_close(stream->io);
 	}

 	return co_err_none;
}

co_error co_stream_read(co_stream stream, void * buffer, unsigned int buffersize)
{
    uint32 data;
    int eos;

    if ( stream == NULL ) {
        return co_err_invalid_arg;
    }

    eos = nr_impc_read(&data, stream->io);
    
    if ( eos ) {
        return co_err_eos;
    }
    else {
        switch ( buffersize ) {
        case 1:
            *(uint8 *)buffer = data;
            break;
        case 2:
            *(uint16 *)buffer = data;
            break;
        default: 
            *(uint32 *)buffer = data;
            break;
        }        
    }

    return co_err_none;
}

int co_stream_read_nb(co_stream stream, void *buffer, unsigned int buffersize)
{
    int data;
    
    if ( stream == NULL ) {
        return 0;
    }
        
    if ( ! nr_impc_read_nb(&data, stream->io) ) {
        return 0;
    }
    else {
        switch (buffersize) {
        case 1:
            *(char *)buffer = data;
            break;
        case 2:
            *(short *)buffer = data;
            break;
        default: 
            *(int *)buffer = data;
            break;
        }
    }

    return 1;
}

co_error co_stream_write(co_stream stream, const void * buffer, unsigned int buffersize)
{
    uint32 data;

    if ( stream == NULL ) {
        return co_err_invalid_arg;
    }

    switch ( buffersize ) {
    case 1:
        data = *(uint8 *)buffer;
        break;
    case 2:
        data = *(uint16 *)buffer;
        break;
    default: 
        data = *(uint32 *)buffer;
        break;
    }

    nr_impc_write(data, stream->io);

    return co_err_none;
}

co_error co_stream_free(co_stream stream)
{
	if ( stream != NULL ) {
		free(stream->name);
		free(stream);
	}
	
	return co_err_none;
}

