/*********************************************************************
* Copyright (c) 2004, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co_register.c: Register functions.
*
*********************************************************************/

#include "system.h"

#include <malloc.h>
#include <string.h>

#include "co.h"

co_register co_register_create(const char * name, co_type type)
{
	co_register reg = malloc(sizeof(co_register_t));	
	if ( reg != NULL ) {
		reg->name = (name != NULL) ? strdup(name) : NULL;
		reg->direction = CO_NO_DIRECTION;
		reg->io = NULL;
	}

	return reg;
}

void co_register_attach(co_register reg, void * io, co_stream_direction dir)
{
	if ( reg != NULL ) {
	    reg->io = io;
    	reg->direction = dir;
    }
}

int32 co_register_get(co_register reg)
{
    int32 value = 0;
    if ( reg != NULL ) {
        (void) nr_impc_read(&value, reg->io);
    }
    return value;
}

void co_register_put(co_register reg, int32 value)
{
    if ( reg != NULL ) {
        nr_impc_write(value, reg->io);
    }
}

co_error co_register_read(co_register reg, void * buffer, unsigned int size)
{
    uint32 data;
    co_error err = co_err_none;

    if ( reg != NULL ) { 
        (void) nr_impc_read(&data, reg->io);
        switch ( size ) {
        case 1:
            *(uint8 *)buffer = data;
            break;
        case 2:
            *(uint16 *)buffer = data;
            break;
        default: 
            *(uint32 *)buffer = data;
            break;
        }
    }
    else {
        err = co_err_invalid_arg;
    }

    return err;
}

co_error co_register_write(co_register reg, const void * buffer, unsigned int size)
{
    uint32 data;
    co_error err = co_err_none;

    if ( reg != NULL ) { 
        switch ( size ) {
        case 1:
            data = *(uint8 *)buffer;
            break;
        case 2:
            data = *(uint16 *)buffer;
            break;
        default: 
            data = *(uint32 *)buffer;
            break;
        }
        nr_impc_write(data, reg->io);
    }
    else {
        err = co_err_invalid_arg;
    }

    return err;
}

