/*******************************************************************************
 * Copyright (c) 2004, Impulse Accelerated Technologies, Inc.
 * All Rights Reserved.
 *
 * nios2_impc.s : Memory-mapped I/O interface routines
 *
 ******************************************************************************/

    /* byte offsets to stream members */
	.equ sf_rxdata,      0  /* Read-only */
	.equ sf_txdata,      0  /* Write-only */
	.equ sf_status,      4  /* Read-only */
	.equ sf_eos,         8  /* Write-only (any value) */

	/* Mask for the ready bit in the status register */
	.equ s_ready_mask,          1
	/* Mask for the EOS bit in the status register */	
	.equ s_eos_mask,            2  
	
		.text

/*******************************************************************************
 *         Name: nr_impc_write
 *  Description: Write a value to an output port
 *        Input: r4: The value
 *               r5: The port
 *       Output: none
 */

		.global	nr_impc_write

nr_impc_write:

stWriteWait:
	ldwio	r8, sf_status(r5)    /* get the status register */
	ori     r9, r8, s_ready_mask /* set the ready bit for comparison */
    bne     r8, r9, stWriteWait  /* loop until not full */
	stwio   r4, sf_txdata(r5)    /* write value */
    ret
	
/*******************************************************************************
 *         Name: nr_impc_read
 *  Description: Read value from input port
 *        Input: r4: Address to receive read data
 *               r5: The port
 *       Output: r2 = EOS?
 */

		.global	nr_impc_read

nr_impc_read:

stReadWait:
	ldwio	r8, sf_status(r5)    /* get the status register */
	ori     r9, r8, s_ready_mask /* set the ready bit for comparison */
    bne     r8, r9, stReadWait   /* loop until not empty */
    andi    r9, r8, s_eos_mask   /* check for EOS */
    cmpeqi  r2, r9, s_eos_mask
    bne     r2, zero, stReadEOS
	ldwio   r9, sf_rxdata(r5)    /* read value */
    stw     r9, 0(r4)            /* store value in output */
stReadEOS:
    ret

/*******************************************************************************
 *         Name: nr_impc_read_nb
 *  Description: Read value from input port without blocking or checking EOS
 * Precondition: Stream has data and is not at EOS
 *        Input: r4: Address to receive read data
 *               r5: The port
 *       Output: r2 = Read successful
 */

		.global	nr_impc_read_nb

nr_impc_read_nb:
	ldwio	r8, sf_status(r5)    /* get the status register */
    cmpeqi  r2, r8, s_ready_mask /* look for ready bit only */
    beq     r2, zero, stReadNotYet
	ldwio   r9, sf_rxdata(r5)    /* read value */
    stw     r9, 0(r4)            /* store value in output */
stReadNotYet:
    ret

/*******************************************************************************
 *         Name: nr_impc_close
 *  Description: Close an output port
 *        Input: r4: The port
 *       Output: none
 */
 
		.global	nr_impc_close

nr_impc_close:

stCloseWait:
	ldwio	r8, sf_status(r4)     /* get the status register */
	ori     r9, r8, s_ready_mask  /* set the ready state for comparison */
    bne     r8, r9, stCloseWait   /* loop until not full/closed */
	stwio   r4, sf_eos(r4)        /* write anything */
	ret

/*******************************************************************************
 *         Name: nr_impc_status
 *  Description: Read status from input port
 *        Input: r4: The port
 *       Output: r2 = Status value
 */

		.global	nr_impc_status

nr_impc_status:
	ldwio   r2, sf_status(r4)  /* read status register */
 	ret

/* end of file */

